/*
 * Functions to start the ethernet system.
 *
 * (C) 2002 Compendium Technologies, Inc.
 * (C) 2002 Softools, Inc.
 * All rights reserved.
 *
 * This software is the proprietary information of Compendium Technologies, Inc.
 * and Softools, Inc. Use is subject to license terms.
 */
#include <Rabbit2000.h>
#include <stdlib.h>
#include <stdarg.h>

#include <rabbitlwip.h>
#include "dns.h"
#include "RTK8019AS.h"
#include "lwip/netif.h"
#include "lwip/api_msg.h"

#ifdef LWIP_DHCP
#include "lwip/dhcp.h"
#endif

extern unsigned char netInited;

/*
 * Flag helps us avoid stepping on configurations that are already set up
 */
unsigned char _ethInitialized = 0;
#ifdef LWIP_DHCP
unsigned char dhcp_ticks = 0;
#endif

near static void
ethernetTimer(void *arg)
{
#ifdef TIMER_DEBUG
    puts("|");
#endif
    tcp_tmr();
    sys_timeout(TCP_TMR_INTERVAL, (sys_timeout_handler) ethernetTimer, NULL);
#ifdef LWIP_DHCP
    dhcp_ticks++;
    if (dhcp_ticks % 5 == 0)
    {
        // Every half second
        dhcp_fine_tmr();
    }
    if (dhcp_ticks % 120 == 0)
    {
        // Every 60 seconds
        dhcp_coarse_tmr();
	dhcp_ticks = 0;
    }
#endif
}

/*
 * Start an ethernet interface, initialize arp and dns.
 * prerequisite: netRecognize().
 */
struct netif * ethernetStart(NetConnectionConfig *config, char useDHCP, ...)
{
    struct netif * interface;
    va_list ap;
    char *nameserver;

    ipset3();
    if (_ethInitialized)
    {
        ipres();
        return NULL;
    }

    // Initialize TCP/UDP/IP system
    // Alternative in threaded environment is to call tcpip_init() to do this

    if (!netInited)
    {
       netInit();
    }

    ip_init();  // From tcpip_thread
    udp_init(); // From tcpip_thread
    tcp_init(); // From tcpip_thread
    arp_init();

    _ethInitialized = 1;
    ipres();

    // Need to initialize timeouts as well
    sys_timeout(TCP_TMR_INTERVAL, (sys_timeout_handler)ethernetTimer, NULL);

    // Set up interface with defaults (in case of DHCP failure) as well
    interface = netif_add(&config->address,
                          &config->netmask,
                          &configuration.defaultGateway,
                          RTK8019AS_if_init,
                          ip_input);
    netif_set_default(interface);

    if (useDHCP)
    {
        /* DHCP/BOOTP initialization */
#ifdef LWIP_DHCP
        struct dhcp_state * client = NULL;
	struct ip_addr noip;
	struct ip_addr mask;
#ifdef DEBUG
        printf("doing DHCP init\n");
#endif
        // Eight tries is about 45 seconds for a timeout
	// TODO: fix there is a discover problem not occurring from other
	// systems.
	noip.addr = IP_ADDR_ANY;
	mask.addr = 0xffffffffL;
        netif_set_ipaddr(interface, &noip);
	//netif_set_netmask(interface, &mask);
        client = dhcp_start(interface);
        // Wait a little bit for the interface to be bound
        while (client->state != DHCP_BOUND && client->tries < 8)
        {
           netProcess();
        }
        // If bound, extract the hostname, domainname, and DNS servers
        // TODO: This doesn't seem to be in there yet
	// If not bound, turn off dhcp engine so the IP doesn't change on us
	// later.
        if (client->state != DHCP_BOUND)
        {
            dhcp_stop(client);
            netif_set_ipaddr(interface, &config->address);
	    //netif_set_netmask(interface, &config->netmask);
        }
#endif
    }
#ifdef DEBUG
    else
    {
        printf("doing static IP init\n");
    }
#endif

    // Swap it once and store so we don't have to do this again.
    //interfaceStats[0].swappedIpAddress = intel(my_ip_addr);

    // Get the list of DNS servers (if there are any) in the variable length
    // argument list.
    va_start(ap, config);
    nameserver = va_arg(ap, char *);
    while(nameserver != NULL)
    {
        struct ip_addr nsip;
#ifdef DEBUG
        printf("adding nameserver\n");
#endif
        if (netStringToIP(nameserver, &nsip))
        {
            dnsAddServer(&nsip);
        }
        nameserver = va_arg(ap, char *);
    }

    return interface;
}
